=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2010-10-07 20:00:40 +0000
+++ drizzled/db.cc	2010-10-24 03:19:07 +0000
@@ -130,7 +130,7 @@
         session->my_ok();
       }
     }
-    else if (not plugin::StorageEngine::createSchema(schema_message)) // Try to create it 
+    else if (not plugin::StorageEngine::createSchema(schema_message, *session)) // Try to create it 
     {
       my_error(ER_CANT_CREATE_DB, MYF(0), schema_message.name().c_str(), errno);
       error= true;

=== modified file 'drizzled/plugin/schema_engine.cc'
--- drizzled/plugin/schema_engine.cc	2010-10-09 01:10:07 +0000
+++ drizzled/plugin/schema_engine.cc	2010-10-24 03:19:07 +0000
@@ -142,26 +142,28 @@
   public unary_function<StorageEngine *, void>
 {
   const drizzled::message::Schema &schema_message;
+  drizzled::Session &session;
 
 public:
 
-  CreateSchema(const drizzled::message::Schema &arg) :
-    schema_message(arg)
+  CreateSchema(const drizzled::message::Schema &arg, drizzled::Session &in_session) :
+    schema_message(arg),
+    session(in_session)
   {
   }
 
   result_type operator() (argument_type engine)
   {
     // @todo eomeday check that at least one engine said "true"
-    (void)engine->doCreateSchema(schema_message);
+    (void)engine->doCreateSchema(schema_message, session);
   }
 };
 
-bool StorageEngine::createSchema(const drizzled::message::Schema &schema_message)
+bool StorageEngine::createSchema(const drizzled::message::Schema &schema_message, Session &session)
 {
   // Add hook here for engines to register schema.
   for_each(StorageEngine::getSchemaEngines().begin(), StorageEngine::getSchemaEngines().end(),
-           CreateSchema(schema_message));
+           CreateSchema(schema_message, session));
 
   return true;
 }

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2010-10-09 01:10:07 +0000
+++ drizzled/plugin/storage_engine.h	2010-10-24 03:19:07 +0000
@@ -332,7 +332,7 @@
   static bool getSchemaDefinition(const drizzled::SchemaIdentifier &identifier, message::SchemaPtr &proto);
   static bool doesSchemaExist(const drizzled::SchemaIdentifier &identifier);
   static const CHARSET_INFO *getSchemaCollation(const drizzled::SchemaIdentifier &identifier);
-  static bool createSchema(const drizzled::message::Schema &schema_message);
+  static bool createSchema(const drizzled::message::Schema &schema_message, drizzled::Session &session);
   static bool dropSchema(const drizzled::SchemaIdentifier &identifier);
   static bool alterSchema(const drizzled::message::Schema &schema_message);
 
@@ -345,8 +345,11 @@
     return false; 
   }
 
-  virtual bool doCreateSchema(const drizzled::message::Schema&)
-  { return false; }
+  virtual bool doCreateSchema(const drizzled::message::Schema&, Session &session)
+  {
+    (void)session;
+    return false;
+  }
 
   virtual bool doAlterSchema(const drizzled::message::Schema&)
   { return false; }

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-10-20 20:26:18 +0000
+++ drizzled/session.h	2010-10-24 03:19:07 +0000
@@ -359,11 +359,23 @@
   Item *free_list;
   memory::Root *mem_root; /**< Pointer to current memroot */
 
-
   memory::Root *getMemRoot()
   {
     return mem_root;
   }
+
+  uint64_t xa_id;
+
+  uint64_t getXaId()
+  {
+    return xa_id;
+  }
+
+  void setXaId(uint64_t in_xa_id)
+  {
+    xa_id= in_xa_id; 
+  }
+
   /**
    * Uniquely identifies each statement object in thread scope; change during
    * statement lifetime.

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2010-10-22 00:51:02 +0000
+++ drizzled/transaction_services.cc	2010-10-24 03:19:07 +0000
@@ -440,6 +440,12 @@
     registerResourceForStatement(session, monitored, engine, resource_manager);
 }
 
+void TransactionServices::allocateNewTransactionId(Session *session)
+{
+  uint64_t xa_id= xa_storage_engine->getNewTransactionId(session);
+  session->setXaId(xa_id);
+}
+
 uint64_t TransactionServices::getCurrentTransactionId(Session *session)
 {
   TransactionContext *trans= &session->transaction.stmt;
@@ -467,7 +473,7 @@
         }
       }
   }
-  return 0;
+  return session->getXaId();
 }
 
 /**
@@ -1009,6 +1015,8 @@
 
   if (should_inc_trx_id)
     trx->set_transaction_id(getCurrentTransactionId(in_session));
+  else 
+    trx->set_transaction_id(0);
 
   trx->set_start_timestamp(in_session->getCurrentTimestamp());
 }

=== modified file 'drizzled/transaction_services.h'
--- drizzled/transaction_services.h	2010-10-22 00:51:02 +0000
+++ drizzled/transaction_services.h	2010-10-24 03:19:07 +0000
@@ -407,6 +407,8 @@
                                       plugin::XaResourceManager *resource_manager);
 
   uint64_t getCurrentTransactionId(Session *session);
+
+  void allocateNewTransactionId(Session *session);
  
   /**
    * DEBUG ONLY.  See plugin::TransactionLog::truncate()

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2010-10-21 01:59:17 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2010-10-24 03:19:07 +0000
@@ -5800,6 +5800,8 @@
                           lex_identified_temp_table ? name2 : NULL,
                           iflags);
 
+  session.setXaId((ib_uint64_t) ut_conv_dulint_to_longlong(trx->id));
+
   if (error) {
     goto cleanup;
   }

=== modified file 'plugin/schema_engine/schema.cc'
--- plugin/schema_engine/schema.cc	2010-10-10 11:41:42 +0000
+++ plugin/schema_engine/schema.cc	2010-10-24 03:19:07 +0000
@@ -30,6 +30,8 @@
 #include "drizzled/data_home.h"
 
 #include "drizzled/internal/my_sys.h"
+#include "drizzled/session.h"
+#include "drizzled/transaction_services.h"
 
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -129,7 +131,7 @@
 }
 
 
-bool Schema::doCreateSchema(const drizzled::message::Schema &schema_message)
+bool Schema::doCreateSchema(const drizzled::message::Schema &schema_message, drizzled::Session &session)
 {
   SchemaIdentifier schema_identifier(schema_message.name());
 
@@ -156,6 +158,9 @@
   }
   mutex.unlock();
 
+  TransactionServices &transaction_services= TransactionServices::singleton();
+  transaction_services.allocateNewTransactionId(&session);
+
   return true;
 }
 

=== modified file 'plugin/schema_engine/schema.h'
--- plugin/schema_engine/schema.h	2010-10-09 01:10:07 +0000
+++ plugin/schema_engine/schema.h	2010-10-24 03:19:07 +0000
@@ -61,7 +61,7 @@
   void doGetSchemaIdentifiers(drizzled::SchemaIdentifiers &set_of_names);
   bool doGetSchemaDefinition(const drizzled::SchemaIdentifier&, drizzled::message::SchemaPtr &proto);
 
-  bool doCreateSchema(const drizzled::message::Schema &schema_message);
+  bool doCreateSchema(const drizzled::message::Schema &schema_message, drizzled::Session &session);
 
   bool doAlterSchema(const drizzled::message::Schema &schema_message);
 

